#ifndef LINT
static char *rcsid="$Id: call.c,v 1.3 2000/02/01 22:05:33 crosser Exp $";
#endif

/*
	"ifmail": FidoNet(r) support package for UN*X platform

	Copyright (c) 1993-1999 Eugene G. Crosser

	You may distribute and/or use for any purpose modified or unmodified
	copies of this software if you preserve the copyright notice above.

	THIS SOFTWARE IS PROVIDED AS IS AND COME WITH NO WARRANTY OF ANY
	KIND, EITHER EXPRESSED OR IMPLIED.  IN NO EVENT WILL THE
	COPYRIGHT HOLDER BE LIABLE FOR ANY DAMAGES RESULTING FROM THE
	USE OF THIS SOFTWARE.

	URL: http://www.average.org/ifmail/
	E-Mail: <crosser@average.org>
*/

/*
	$Log: call.c,v $
	Revision 1.3  2000/02/01 22:05:33  crosser
	janus patch
	Txy flag processing
	
	Revision 1.2  1999/12/28 16:01:53  crosser
	Move to syslog
	Portability for HP/UX
	
	Revision 1.1.1.1  1999/08/29 22:53:46  crosser
	Ifmail
	
*/

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <sys/types.h>
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif
#include <stdio.h>
#ifdef STDC_HEADERS
# include <stdlib.h>
# include <string.h>
#else
# ifdef HAVE_STRINGS_H
#  include <strings.h>
# endif
#endif
#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#include "lutil.h"
#include "xutil.h"
#include "ftn.h"
#include "nodelist.h"
#include "rdconfig.h"
#include "session.h"
#include "callstat.h"

extern int forcedcalls;
extern char *forcedphone;
extern char *forcedline;
#if defined(HAVE_TCP) || defined(HAVE_TERM)
extern char *inetaddr;
#endif

extern char *get_modem_string(modem_string*,node*);
extern int openport(char*,int);
#if defined(HAVE_TCP) || defined(HAVE_TERM)
extern int opentcp(char*);
extern void closetcp(void);
#endif
extern void localport(void);
extern void nolocalport(void);
extern void closeport(void);
extern int dialphone(node*,char*);
extern int hangup(node*);
extern void rdoptions(node*);
extern int nodelock(faddr*);
extern int nodeulock(faddr*);
extern void setproctitle(char*);
extern int istxy(node *);

int checkretry(st)
callstat *st;
{
	return 0;
	/* check retries and time; rc=1 - not reached, rc=2 - undialable */
}

int call(addr)
faddr *addr;
{
	node *nlent;
	int rc=1;
	int speed;
	char *portlist,*p,*q;
	callstat *st;
	char cbuf[128];

	if ((nlent=getnlent(addr)) == NULL)
	{
		syslog(LOG_ERR,"cannot call %s: fatal in nodelist lookup",
			ascfnode(addr,0x1f));
		putstatus(addr,0,6);
		return 6;
	}

	st=getstatus(addr);
	if (!forcedcalls) {
		if ((rc=checkretry(st))) {
			syslog(LOG_INFO,"cannot call %s: %s",
				ascfnode(addr,0x1f),
				(rc==1)?"retry time not reached"
				:"node undialable");
			return 5;
		}
		if (dialdelay) sleep(dialdelay);
	}

	if (nodelock(addr))
	{
		syslog(LOG_INFO,"system %s locked",ascinode(addr,0x1f));
		putstatus(addr,0,4);
		return 4;
	}

	rdoptions(nlent);

	inbound=protinbound; /* master sessions are secure */

	sprintf(cbuf,"ifcico calling %s (%s %s)",
		ascfnode(&(nlent->addr),0x1f),
#if defined(HAVE_TCP) || defined(HAVE_TERM)
		inetaddr?"addr":
#endif
				"phone",
#if defined(HAVE_TCP) || defined(HAVE_TERM)
		inetaddr?inetaddr:
#endif
			forcedphone?forcedphone:
				nlent->phone?nlent->phone:"<none>");
	setproctitle(cbuf);

	if ((nlent->phone || forcedphone
#if defined(HAVE_TCP) || defined(HAVE_TERM)
					 || inetaddr
#endif
							) &&
	    (forcedcalls || 
	     (((nlent->pflag & (NL_DUMMY|NL_DOWN|NL_HOLD|NL_PVT)) == 0) && 
	      ((localoptions & NOCALL) == 0) &&
	      (istxy(nlent) || (localoptions & NOTXY)))))
	{
		syslog(LOG_INFO,"calling %s (%s, phone %s)",ascfnode(addr,0x1f),
			nlent->name,nlent->phone?nlent->phone:forcedphone);
#if defined(HAVE_TCP) || defined(HAVE_TERM)
		if (inetaddr)
		{
			rc=opentcp(inetaddr);
			if (rc)
			{
				syslog(LOG_INFO,"cannot connect %s",inetaddr);
				nodeulock(addr);
				putstatus(addr,1,2);
				return 2;
			}
		}
		else
#endif
		     if (forcedline)
		{
			p=forcedline;
			if ((q=strchr(p,':')))
			{
				*q++='\0';
				if ((*q == 'l') || (*q == 'L'))
					speed=atoi(++q);
				else
				{
					speed=atoi(q);
					if (nlent->speed < speed)
						speed=nlent->speed;
				}
			}
			else speed=0;
			setnamesfx(p);
			rc=openport(p,speed);
			if (rc)
			{
				syslog(LOG_INFO,"cannot open port %s",p);
				nodeulock(addr);
				putstatus(addr,0,1);
				return 1;
			}
		}
		else
		{
			if ((portlist=xstrcpy(get_modem_string(modemport,nlent))) == NULL)
			{
				syslog(LOG_ERR,"no matching ports defined");
				nodeulock(addr);
				putstatus(addr,0,9);
				return 9;
			}
			for (rc=1,p=strtok(portlist," \t,");
				rc && p;
				p=strtok(NULL," \t,"))
			{
				if ((q=strrchr(p,':')))
				{
					*q++='\0';
					if ((*q == 'l') || (*q == 'L'))
					{
						speed=atoi(++q);
					}
					else
					{
						speed=atoi(q);
						if (nlent->speed < speed)
							speed=nlent->speed;
					}
				}
				else speed=0;
				setnamesfx(p);
				rc=openport(p,speed);
			}
			if (rc)
			{
				syslog(LOG_INFO,"no free matching ports");
				free(portlist);
				nodeulock(addr);
				putstatus(addr,0,1);
				return 1;
			}
			free(portlist);
		}
#if defined(HAVE_TCP) || defined(HAVE_TERM)
		if (!inetaddr)
#endif
		if ((rc=dialphone(nlent,forcedphone?forcedphone:nlent->phone)))
		{
			syslog(LOG_INFO,"dial failed");
			nodeulock(addr);
			rc+=1; /* rc=2 - dial fail, rc=3 - could not reset */
		}

		if (rc == 0)
		{
#if defined(HAVE_TCP) || defined(HAVE_TERM)
			if (!inetaddr)
#endif
				nolocalport();
			rc=session(addr,nlent,SESSION_MASTER,
				SESSION_UNKNOWN,NULL);
			if (rc) rc=abs(rc)+10;
		}
#if defined(HAVE_TCP) || defined(HAVE_TERM)
		if (inetaddr)
		{
			closetcp();
		}
		else
#endif
		{
			localport();
			hangup(nlent);
			closeport();
		}
	}
	else
	{
		syslog(LOG_INFO,"cannot call %s (%s, phone %s)",
			ascfnode(addr,0x1f),S(nlent->name),
			S(nlent->phone));
		if ((nlent->phone || forcedphone
#if defined(HAVE_TCP) || defined(HAVE_TERM)
						 || inetaddr
#endif
								))
			rc=8;
		else rc=7;
		nodeulock(addr);
	}

	if ((rc == 2) || (rc == 30))
		putstatus(addr,1,rc);
	else
		putstatus(addr,0,rc);
	return rc;
}
