#ifndef LINT
static char *rcsid="$Id: opentcp.c,v 1.6 2000/01/01 21:23:32 crosser Exp $";
#endif

/*
	"ifmail": FidoNet(r) support package for UN*X platform

	Copyright (c) 1993-1999 Eugene G. Crosser

	You may distribute and/or use for any purpose modified or unmodified
	copies of this software if you preserve the copyright notice above.

	THIS SOFTWARE IS PROVIDED AS IS AND COME WITH NO WARRANTY OF ANY
	KIND, EITHER EXPRESSED OR IMPLIED.  IN NO EVENT WILL THE
	COPYRIGHT HOLDER BE LIABLE FOR ANY DAMAGES RESULTING FROM THE
	USE OF THIS SOFTWARE.

	URL: http://www.average.org/ifmail/
	E-Mail: <crosser@average.org>
*/

/*
	$Log: opentcp.c,v $
	Revision 1.6  2000/01/01 21:23:32  crosser
	set user/group, chat responce logging
	
	Revision 1.5  1999/12/29 23:16:54  crosser
	make syslog and debug use %m for error message
	
	Revision 1.4  1999/12/28 16:01:54  crosser
	Move to syslog
	Portability for HP/UX
	
	Revision 1.3  1999/12/28 06:37:28  crosser
	Make TCP slave/daemon and master modes work
	
	Revision 1.2  1999/09/12 23:24:30  crosser
	Clean tests
	Work in progress on daemon mode
	
	Revision 1.1.1.1  1999/08/29 22:53:46  crosser
	Ifmail
	
*/

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <sys/types.h>
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif
#include <stdio.h>
#ifdef STDC_HEADERS
# include <stdlib.h>
# include <string.h>
#else
# ifdef HAVE_STRINGS_H
#  include <strings.h>
# endif
#endif
#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#ifdef HAVE_TCP

#include <sys/param.h>
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <errno.h>
#include <signal.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "lutil.h"

#define FIDOPORT 60179		/* my birthday */

extern void linedrop(int);

int opentcp(char*);
void closetcp(void);
void setproctitle(char*);
int answer(char *);

extern int h_errno;

static int fd=-1;
extern int f_flags;

/* opentcp() was rewritten by Martin Junius */

int opentcp(name)
char *name;
{
	struct servent *se;
	struct hostent *he;
	struct protoent *proto;
	int a1,a2,a3,a4;
	char *errmsg;
	char *portname;
	int fd;
	short portnum;
	int protonum;
	struct sockaddr_in server;

	debug(18,"try open tcp connection to %s",S(name));

	server.sin_family=AF_INET;

	if ((proto=getprotobyname("tcp"))) protonum=proto->p_proto;
	else protonum=6;
	if ((portname=strchr(name,':')))
	{
		*portname++='\0';
		if ((portnum=atoi(portname)))
			server.sin_port=htons(portnum);
		else if ((se=getservbyname(portname,"tcp")))
			server.sin_port=se->s_port;
		else server.sin_port=htons(FIDOPORT);
	}
	else
	{
		if ((se=getservbyname("fido","tcp")))
			server.sin_port=se->s_port;
		else server.sin_port=htons(FIDOPORT);
	}

	if (sscanf(name,"%d.%d.%d.%d",&a1,&a2,&a3,&a4) == 4)
		server.sin_addr.s_addr=inet_addr(name);
	else if ((he=gethostbyname(name)))
		memcpy(&server.sin_addr,he->h_addr,he->h_length);
	else
	{
		switch (h_errno)
		{
		case HOST_NOT_FOUND:	errmsg="Authoritative: Host not found"; break;
		case TRY_AGAIN:		errmsg="Non-Authoritive: Host not found"; break;
		case NO_RECOVERY:	errmsg="Non recoverable errors"; break;
		default:		errmsg="Unknown error"; break;
		}
		syslog(LOG_INFO,"no IP address for %s: %s\n",name,errmsg);
		return -1;
	}

	debug(18,"trying %s at port %d",
		inet_ntoa(server.sin_addr),(int)ntohs(server.sin_port));

	signal(SIGPIPE,linedrop);
	fflush(stdin);
	fflush(stdout);
	setbuf(stdin,NULL);
	setbuf(stdout,NULL);
	if ((fd=socket(AF_INET,SOCK_STREAM,protonum)) < 0)
	{
		syslog(LOG_ERR,"cannot create socket: %m");
		return -1;
	}
	if (dup2(fd,0) < 0) {
		syslog(LOG_ERR,"cannot dup socket to 0: %m");
		return -1;
	}
	if (dup2(fd,1) < 0)
	{
		syslog(LOG_ERR,"cannot dup socket to 1: %m");
		return -1;
	}
	clearerr(stdin);
	clearerr(stdout);
	if (connect(fd,(struct sockaddr *)&server,sizeof(server)) == -1)
	{
		syslog(LOG_INFO,"cannot connect %s: %m",
			inet_ntoa(server.sin_addr));
		return -1;
	}

	f_flags=0;

	syslog(LOG_INFO,"established TCP connection with %s",inet_ntoa(server.sin_addr));
	return 0;
}

void closetcp(void)
{
	shutdown(fd,2);
	signal(SIGPIPE,SIG_DFL);
}

void listentcp(name)
char *name;
{
	struct servent *se;
	struct hostent *he;
	struct protoent *proto;
	int a1,a2,a3,a4;
	char *errmsg;
	char *portname;
	int fd,msgfd;
	int protonum;
	short portnum;
	struct sockaddr_in server,from;
	int flen;
	int pid,maxrc,tries,on=1;

	debug(18,"try listen on %s",S(name));

	server.sin_family=AF_INET;

	if ((proto=getprotobyname("tcp"))) protonum=proto->p_proto;
	else protonum=6;
	if (name && ((portname=strchr(name,':')))) {
		*portname++='\0';
		if ((portnum=atoi(portname)))
			server.sin_port=htons(portnum);
		else if ((se=getservbyname(portname,"tcp")))
			server.sin_port=se->s_port;
		else server.sin_port=htons(FIDOPORT);
	} else {
		if ((se=getservbyname("fido","tcp")))
			server.sin_port=se->s_port;
		else server.sin_port=htons(FIDOPORT);
	}

	if (name && *name) {
		if (sscanf(name,"%d.%d.%d.%d",&a1,&a2,&a3,&a4) == 4)
			server.sin_addr.s_addr=inet_addr(name);
		else if ((he=gethostbyname(name)))
			memcpy(&server.sin_addr,he->h_addr,he->h_length);
		else {
			switch (h_errno) {
			case HOST_NOT_FOUND:	errmsg="Authoritative: Host not found"; break;
			case TRY_AGAIN:		errmsg="Non-Authoritive: Host not found"; break;
			case NO_RECOVERY:	errmsg="Non recoverable errors"; break;
			default:		errmsg="Unknown error"; break;
			}
			syslog(LOG_INFO,"no IP address for %s: %s\n",name,errmsg);
			return;
		}
	} else server.sin_addr.s_addr=INADDR_ANY;

	debug(18,"going to listen on %s at port %d",
		inet_ntoa(server.sin_addr),(int)ntohs(server.sin_port));

	if ((fd=socket(AF_INET,SOCK_STREAM,protonum)) < 0) {
		syslog(LOG_ERR,"cannot create socket (got %d, expected 0: %m");
		return;
	}
	if (setsockopt(fd,SOL_SOCKET,SO_REUSEADDR,
			(char*)&on,sizeof(on)) == -1) {
		syslog(LOG_ERR,"setsockopt SO_REUSEADDR failed: %m");
		return;
	}
	for (tries=0;;tries++) {
		if (bind(fd,(struct sockaddr*)&server,
						sizeof(server)) < 0) {
			if ((errno == EADDRINUSE) && (tries < 10)) {
				debug(18,"bind attempt %d failed",tries);
				sleep(tries);
				continue;
			}
			syslog(LOG_ERR,"bind failure on %s at port %d: %m",
				inet_ntoa(server.sin_addr),
				(int)ntohs(server.sin_port));
			return;
		} else break;
	}
	debug(18,"bind success, going to listen");
	if (listen(fd,5) < 0) {
		syslog(LOG_ERR,"listen failure on %s at port %d: %m",
			inet_ntoa(server.sin_addr),
			(int)ntohs(server.sin_port));
		return;
	}
	setproctitle("-ifcico listening");

	syslog(LOG_INFO,"Listening on %s at port %d",
		inet_ntoa(server.sin_addr),
		(int)ntohs(server.sin_port));

	for (;;) {
		debug(18,"waiting on accept fd=%d",fd);
		flen=sizeof(from);
		msgfd=accept(fd,(struct sockaddr*)&from,&flen);
		if (msgfd < 0) {
			syslog(LOG_ERR,"accept failure: %m");
			continue;
		}
		syslog(LOG_INFO,"accepted TCP connection from %s port %d fd %d",
			inet_ntoa(from.sin_addr),
			(int)ntohs(from.sin_port),
			msgfd);
		if ((pid=fork()) < 0) {
			syslog(LOG_ERR,"fork failure: %m");
			close(msgfd);
			continue;
		} else if (pid) {
			debug(18,"forked child %d for connection on fd %d",
				pid,msgfd);
			close(msgfd);
			continue;
		} else {
			debug(18,"child %d ready for connection on fd %d",
				getpid(),msgfd);
			setproctitle("-ifcico answering");
			close(fd);
			if (dup2(msgfd,0) != 0) {
				syslog(LOG_ERR,"cannot dup socket to 0: %m");
				return;
			}
			if (dup2(msgfd,1) != 1) {
				syslog(LOG_ERR,"cannot dup socket to 1: %m");
				return;
			}
			if (msgfd > 2) close(msgfd);
			debug(18,"ready to answer");
			maxrc=answer(NULL);
			syslog(LOG_INFO,"1 of 1 calls, maxrc=%d",maxrc);
			exit(maxrc);
		}
	}
	return; /* not reached */
}

#endif
