#ifndef LINT
static char *rcsid="$Id: ifdbm.c,v 1.6 1999/12/29 23:16:54 crosser Exp $";
#endif

/*
	"ifmail": FidoNet(r) support package for UN*X platform

	Copyright (c) 1993-1999 Eugene G. Crosser

	You may distribute and/or use for any purpose modified or unmodified
	copies of this software if you preserve the copyright notice above.

	THIS SOFTWARE IS PROVIDED AS IS AND COME WITH NO WARRANTY OF ANY
	KIND, EITHER EXPRESSED OR IMPLIED.  IN NO EVENT WILL THE
	COPYRIGHT HOLDER BE LIABLE FOR ANY DAMAGES RESULTING FROM THE
	USE OF THIS SOFTWARE.

	URL: http://www.average.org/ifmail/
	E-Mail: <crosser@average.org>
*/

/*
	$Log: ifdbm.c,v $
	Revision 1.6  1999/12/29 23:16:54  crosser
	make syslog and debug use %m for error message
	
	Revision 1.5  1999/12/28 16:01:54  crosser
	Move to syslog
	Portability for HP/UX
	
	Revision 1.4  1999/11/04 18:34:47  crosser
	better job in finding ndbm
	
	Revision 1.3  1999/11/03 07:06:08  crosser
	Check for ndbm in more places
	
	Revision 1.2  1999/08/30 22:56:03  crosser
	dbm checks et al
	
	Revision 1.1.1.1  1999/08/29 22:53:45  crosser
	Ifmail
	
*/

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <sys/types.h>
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif
#include <stdio.h>
#ifdef STDC_HEADERS
# include <stdlib.h>
# include <string.h>
#else
# ifdef HAVE_STRINGS_H
#  include <strings.h>
# endif
#endif
#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#include <errno.h>

#include "lutil.h"
#include "rdconfig.h"

#if defined(HAVE_NDBM_H)
#include <ndbm.h>
#elif defined(HAVE_GDBM_NDBM_H)
#include <gdbm/ndbm.h>
#elif defined(HAVE_DB1_NDBM_H)
#include <db1/ndbm.h>
#else
#include <dbm.h>
#endif

#ifdef HAVE_NDBM
static DBM *alias_db = NULL;
#endif

static int opened=0;

static int init(void)
{
#ifndef HAVE_BSD_DB
	char buf[128];
	struct stat stbuf;
	FILE *fp;
#endif
	int tries;

	if (database == NULL) return -1;
	if (opened == -1) return -1;
	if (opened) return 0;
#ifndef HAVE_BSD_DB
	sprintf(buf,"%s.dir",database);
	if (stat(buf,&stbuf) != 0)
	{
		sprintf(buf,"%s.dir",database);
		fp=fopen(buf,"a");
		if (fp) fclose(fp);
		sprintf(buf,"%s.pag",database);
		fp=fopen(buf,"a");
		if (fp) fclose(fp);
	}
#endif
	for (tries=0;(1);tries++)
	{
#ifdef HAVE_NDBM
		if ((alias_db=dbm_open(database,O_RDWR | O_CREAT,0600)) != NULL)
#else
		if (dbminit(database) == 0)
#endif
		{
			opened = 1;
			return 0;
		}
		else if ((tries > 5) || (errno != EAGAIN))
		{
			syslog(LOG_ERR,"could not open alias database: %m");
			opened = -1;
			return -1;
		}
		else
			sleep(2);
	}
}

int registrate(freename,address)
char *freename,*address;
{
	datum key,val;
	char buf[128],*p,*q;
	int first;

	if (init()) return 1;

	strncpy(buf,freename,sizeof(buf)-1);
	first=1;
	for (p=buf,q=buf;*p;p++) switch (*p)
	{
	case '.':	*p=' '; /* fallthrough */
	case ' ':	if (first)
			{
				*(q++)=*p;
				first=0;
			}
			break;
	default:	*(q++)=*p;
			first=1;
			break;
	}
	*q='\0';
	debug(6,"registrate \"%s\" \"%s\"",S(buf),S(address));
	key.dptr=buf;
	key.dsize=strlen(buf);
#ifdef HAVE_NDBM
	val=dbm_fetch(alias_db,key);
#else
	val=fetch(key);
#endif
	if (val.dptr) return 0; /* already present */
	else
	{
		val.dptr=address;
		val.dsize=strlen(address);
#ifdef HAVE_NDBM
		if (dbm_store(alias_db,key,val,0) < 0)
#else
		if (store(key,val) != 0)
#endif
		{
			syslog(LOG_ERR,"cannot store: \"%s\" \"%s\": %m",
				S(buf),S(address));
			return 1;
		}
		else
		{
			syslog(LOG_INFO,"registered \"%s\" as \"%s\"",S(buf),S(address));
			return 0;
		}
	}
}

char *lookup(freename)
char *freename;
{
	datum key,val;
	static char buf[128],*p,*q;
	int sz,first;

	if (init()) return NULL;

	strncpy(buf,freename,sizeof(buf)-1);
	first=1;
	for (p=buf,q=buf;*p;p++) switch (*p)
	{
	case '.':	*p=' '; /* fallthrough */
	case ' ':	if (first)
			{
				*(q++)=*p;
				first=0;
			}
			break;
	default:	*(q++)=*p;
			first=1;
			break;
	}
	*q='\0';
	debug(6,"lookup \"%s\"",S(freename));
	key.dptr=buf;
	key.dsize=strlen(buf);
#ifdef HAVE_NDBM
	val=dbm_fetch(alias_db,key);
#else
	val=fetch(key);
#endif
	if (val.dptr)
	{
		sz=val.dsize;
		if (sz > (sizeof(buf)-1)) sz=sizeof(buf)-1;
		strncpy(buf,val.dptr,sz);
		buf[sz]='\0';
		debug(6,"found: \"%s\"",buf);
		return(buf);
	}
	else return NULL;
}

void close_alias_db(void)
{
	if (opened != 1) return;
#ifdef HAVE_DBM_CLOSE
#ifdef HAVE_NDBM
	dbm_close(alias_db);
#else
	dbmclose();
#endif
#endif
	opened=0;
}
