#ifndef LINT
static char *rcsid="$Id: ifunpack.c,v 1.5 1999/12/29 23:16:54 crosser Exp $";
#endif

/*
	"ifmail": FidoNet(r) support package for UN*X platform

	Copyright (c) 1993-1999 Eugene G. Crosser

	You may distribute and/or use for any purpose modified or unmodified
	copies of this software if you preserve the copyright notice above.

	THIS SOFTWARE IS PROVIDED AS IS AND COME WITH NO WARRANTY OF ANY
	KIND, EITHER EXPRESSED OR IMPLIED.  IN NO EVENT WILL THE
	COPYRIGHT HOLDER BE LIABLE FOR ANY DAMAGES RESULTING FROM THE
	USE OF THIS SOFTWARE.

	URL: http://www.average.org/ifmail/
	E-Mail: <crosser@average.org>
*/

/*
	$Log: ifunpack.c,v $
	Revision 1.5  1999/12/29 23:16:54  crosser
	make syslog and debug use %m for error message
	
	Revision 1.4  1999/12/28 16:01:54  crosser
	Move to syslog
	Portability for HP/UX
	
	Revision 1.3  1999/11/26 18:11:36  crosser
	Tune for sunos4
	
	Revision 1.2  1999/08/30 16:43:16  crosser
	Make the thing at least compile.
	
	Revision 1.1.1.1  1999/08/29 22:53:45  crosser
	Ifmail
	
*/

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <sys/types.h>
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif
#include <stdio.h>
#ifdef STDC_HEADERS
# include <stdlib.h>
# include <string.h>
#else
# ifdef HAVE_STRINGS_H
#  include <strings.h>
# endif
#endif
#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif
#ifndef OPTARG_DEFINED
#include <getopt.h>
#endif
#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif
#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#include "cspace.h"
#include "directory.h"
#ifdef HAVE_SYSLOG
#include <syslog.h>
#endif
#include "mygetopt.h"
#include "xutil.h"
#include "lutil.h"
#include "rdconfig.h"
#include "trap.h"

#define UNPACK_FACTOR 300
#define TOSS_FACTOR 120
#ifndef NEWSSPOOL
#define NEWSSPOOL "/var/spool/news"
#endif

extern char* log_name;

static int unpack(char *);
static int toss(char *);
extern int f_lock(char *);
extern void funlock(int);
extern int execute(char *,char *,char *,char *,char *,char *);
extern char *unpacker(char *);

void usage(void)
{
	confusage("");
}

int main(argc,argv)
int argc;
char *argv[];
{
	int c;
	int rc=0,maxrc=0;
	int files=0,files_ok=0,packets=0,packets_ok=0;
	DIR *dp;
	struct dirent *de;

#if defined(HAVE_SYSLOG) && defined(MAILLOG)
        logfacility=MAILLOG;
#endif

	setmyname(argv[0]);
	catch(myname);
	while ((c=getopt(argc,argv,"hx:I:")) != -1)
	if (confopt(c,optarg)) switch (c)
	{
		default:	usage(); exit(1);
	}

	if (readconfig())
	{
		fprintf(stderr,"Error getting configuration, aborting\n");
		exit(1);
	}

	if (chdir(protinbound) == -1)
	{
		syslog(LOG_ERR,"Error changing to directory %s: %m",S(protinbound));
		exit(1);
	}

	if ((dp=opendir(protinbound)) == NULL)
	{
		syslog(LOG_ERR,"Error opening directory %s: %m",S(protinbound));
		exit(1);
	}

	umask(066);

	while((de=readdir(dp)))
	if ((strlen(de->d_name) == 12) &&
	    ((strncasecmp(de->d_name+8,".su",3) == 0) ||
	     (strncasecmp(de->d_name+8,".mo",3) == 0) ||
	     (strncasecmp(de->d_name+8,".tu",3) == 0) ||
	     (strncasecmp(de->d_name+8,".we",3) == 0) ||
	     (strncasecmp(de->d_name+8,".th",3) == 0) ||
	     (strncasecmp(de->d_name+8,".fr",3) == 0) ||
	     (strncasecmp(de->d_name+8,".sa",3) == 0)))
	{
		files++;
		if (checkspace(protinbound,de->d_name,UNPACK_FACTOR))
			if ((rc=unpack(de->d_name)) == 0) files_ok++;
			else syslog(LOG_ERR,"Error unpacking file %s",de->d_name);
		else syslog(LOG_INFO,"Insufficient space to unpack file %s",de->d_name);
		if (rc > maxrc) maxrc=rc;
	}

	rewinddir(dp);

	while((de=readdir(dp)))
	if ((strlen(de->d_name) == 12) &&
	    (strncasecmp(de->d_name+8,".pkt",4) == 0))
	{
		packets++;
		if (checkspace(NEWSSPOOL,de->d_name,TOSS_FACTOR))
			if ((rc=toss(de->d_name)) == 0) packets_ok++;
			else syslog(LOG_ERR,"Error tossing packet %s",de->d_name);
		else syslog(LOG_INFO,"Insufficient space to toss packet %s",de->d_name);
		if (rc > maxrc) maxrc=rc;
	}

	closedir(dp);

	if (files || packets)
		syslog(LOG_INFO,"processed %d of %d files, %d of %d packets, rc=%d",
			files_ok,files,packets_ok,packets,maxrc);

	return maxrc;
}

int unpack(fn)
char *fn;
{
	char newname[16];
	char *cmd;
	int rc,ld;

	if ((cmd=unpacker(fn)) == NULL) return 1;
	if ((ld=f_lock(fn)) == -1) return 1;
	rc=execute(cmd,fn,(char *)NULL,"/dev/null",log_name,log_name);
	if (rc == 0) unlink(fn);
	else
	{
		strncpy(newname,fn,sizeof(newname)-1);
		strcpy(newname+8,".bad");
		rename(fn,newname);
	}
	funlock(ld);
	return rc;
}

int toss(fn)
char *fn;
{
	int rc,ld;
	char newname[16];
	char *cmd,tmpb[32],*p;
	int i;

	if ((ld=f_lock(fn)) == -1) return 1;
	p=tmpb;
	*p='\0';
	for (i=0;i<26;i++)
		if (verbose & (1<<i))
			*p++='a'+i;
	*p='\0';
	cmd=xstrcpy(iftoss);
	if (tmpb[0])
	{
		cmd=xstrcat(cmd," -x ");
		cmd=xstrcat(cmd,tmpb);
	}
	cmd=xstrcat(cmd," -I ");
	cmd=xstrcat(cmd,configname);
	rc=execute(cmd,(char *)NULL,(char *)NULL,fn,log_name,log_name);
	free(cmd);
	if (rc == 0) unlink(fn);
	else
	{
		strncpy(newname,fn,sizeof(newname)-1);
		strcpy(newname+8,".bad");
		rename(fn,newname);
	}
	funlock(ld);
	return rc;
}
