#ifndef LINT
static char *rcsid="$Id: cspace.c,v 1.3 1999/12/29 23:16:54 crosser Exp $";
#endif

/*
	"ifmail": FidoNet(r) support package for UN*X platform

	Copyright (c) 1993-1999 Eugene G. Crosser

	You may distribute and/or use for any purpose modified or unmodified
	copies of this software if you preserve the copyright notice above.

	THIS SOFTWARE IS PROVIDED AS IS AND COME WITH NO WARRANTY OF ANY
	KIND, EITHER EXPRESSED OR IMPLIED.  IN NO EVENT WILL THE
	COPYRIGHT HOLDER BE LIABLE FOR ANY DAMAGES RESULTING FROM THE
	USE OF THIS SOFTWARE.

	URL: http://www.average.org/ifmail/
	E-Mail: <crosser@average.org>
*/

/*
	$Log: cspace.c,v $
	Revision 1.3  1999/12/29 23:16:54  crosser
	make syslog and debug use %m for error message
	
	Revision 1.2  1999/12/28 16:01:55  crosser
	Move to syslog
	Portability for HP/UX
	
	Revision 1.1.1.1  1999/08/29 22:53:45  crosser
	Ifmail
	
*/

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <sys/types.h>
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif
#include <stdio.h>
#ifdef STDC_HEADERS
# include <stdlib.h>
# include <string.h>
#else
# ifdef HAVE_STRINGS_H
#  include <strings.h>
# endif
#endif
#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#include <sys/types.h>
#include <sys/param.h>
#include <sys/stat.h>
#include "lutil.h"
#include "cspace.h"

#ifndef ULONG_MAX
#define ULONG_MAX 4294967295
#endif

#if defined(HAVE_STATFS) | defined(HAVE_STATVFS)

#if defined(HAVE_STATFS)
#if defined(STATFS_IN_VFS_H)
#include <sys/vfs.h>
#elif defined(STATFS_IN_STATFS_H)
#include <sys/statfs.h>
#elif defined(STATFS_IN_STATVFS_H)
#include <sys/statvfs.h>
#elif defined(STATFS_IN_MOUNT_H)
#include <sys/mount.h>
#else
#error No include for statfs() call defined
#endif
#elif defined(HAVE_STATVFS)
#include <sys/statvfs.h>
#endif

int checkspace(dir,fn,factor)
char *dir,*fn;
int factor;
{
	struct stat st;

#ifdef HAVE_STATVFS
	struct statvfs sfs;

	if ((stat(fn,&st) != 0) || (statvfs(dir,&sfs) != 0))
#else
	struct statfs sfs;

#ifdef SCO_STYLE_STATFS
	if ((stat(fn,&st) != 0) || (statfs(dir,&sfs,sizeof(sfs),0) != 0))
#else
	if ((stat(fn,&st) != 0) || (statfs(dir,&sfs) != 0))
#endif
#endif
	{
		syslog(LOG_ERR,"cannot stat \"%s\" or statfs \"%s\": %m, assume enough space",
			S(fn),S(dir));
		return 1;
	}
	if ((((st.st_size/sfs.f_bsize+1)*factor)/100L) > sfs.f_bfree)
	{
		syslog(LOG_INFO,"Only %lu %lu-byte blocks left on device where %s is located",
			sfs.f_bfree,sfs.f_bsize,S(dir));
		return 0;
	}
	return 1;
}

unsigned long freespace(dir)
char *dir;
{
#ifdef HAVE_STATVFS
	struct statvfs sfs;

	if (statvfs(dir,&sfs) != 0)
#else
	struct statfs sfs;

#ifdef SCO_STYLE_STATFS
	if (statfs(dir,&sfs,sizeof(sfs),0) != 0)
#else
	if (statfs(dir,&sfs) != 0)
#endif
#endif
	{
		syslog(LOG_ERR,"cannot statfs \"%s\": %m, assume plenty space",S(dir));
		return ULONG_MAX;
	}

	if (sfs.f_bfree > (ULONG_MAX / sfs.f_bsize))
		return ULONG_MAX;
	else
		return (sfs.f_bsize * sfs.f_bfree);
}

#else /* does not have stat[v]fs */

int checkspace(dir,fn,factor)
char *dir,*fn;
int factor;
{
	return 1; /* Assume space is enough */
}

unsigned long freespace(dir)
char *dir;
{
	return ULONG_MAX; /* assume there is plenty space */
}

#endif
