#ifndef LINT
static char *rcsid="$Id: nodelock.c,v 1.2 1999/12/28 16:01:55 crosser Exp $";
#endif

/*
	"ifmail": FidoNet(r) support package for UN*X platform

	Copyright (c) 1993-1999 Eugene G. Crosser

	You may distribute and/or use for any purpose modified or unmodified
	copies of this software if you preserve the copyright notice above.

	THIS SOFTWARE IS PROVIDED AS IS AND COME WITH NO WARRANTY OF ANY
	KIND, EITHER EXPRESSED OR IMPLIED.  IN NO EVENT WILL THE
	COPYRIGHT HOLDER BE LIABLE FOR ANY DAMAGES RESULTING FROM THE
	USE OF THIS SOFTWARE.

	URL: http://www.average.org/ifmail/
	E-Mail: <crosser@average.org>
*/

/*
	$Log: nodelock.c,v $
	Revision 1.2  1999/12/28 16:01:55  crosser
	Move to syslog
	Portability for HP/UX
	
	Revision 1.1.1.1  1999/08/29 22:53:45  crosser
	Ifmail
	
*/

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <sys/types.h>
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif
#include <stdio.h>
#ifdef STDC_HEADERS
# include <stdlib.h>
# include <string.h>
#else
# ifdef HAVE_STRINGS_H
#  include <strings.h>
# endif
#endif
#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include <sys/stat.h>
#include "xutil.h"
#include "lutil.h"
#include "ftn.h"

#ifdef HAVE_BSY

#if defined(SHORT_PID_T)
#define pid_t short
#elif defined(INT_PID_T)
#define pid_t int
#endif

extern char *bsyname(faddr *);
extern void mkdirs(char*);

int nodelock(addr)
faddr *addr;
{
	char *fn,*tfn,*p;
	char tmp[16];
	FILE *fp;
	pid_t pid,mypid;
	int tmppid,sverr;

	debug(4,"try locking node %s",ascfnode(addr,0x1f));

	fn=bsyname(addr);
	tfn=xstrcpy(fn);
	if ((p=strrchr(tfn,'/'))) *++p='\0';
	mypid=getpid();
	sprintf(tmp,"aa%d",mypid);
	tfn=xstrcat(tfn,tmp);
	mkdirs(tfn);
	if ((fp=fopen(tfn,"w")) == NULL) {
		syslog(LOG_ERR,"$cannot open tmp file for bsy lock \"%s\"",tfn);
		free(tfn);
		return 1;
	}
	fprintf(fp,"%10d\n",mypid);
	fclose(fp);
	chmod(tfn,0444);
	if (link(tfn,fn) == 0) {
		debug(4,"created lock OK");
		unlink(tfn);
		free(tfn);
		return 0;
	} else {
		sverr=errno;
		debug(4,".bsy file present, check staleness");
	}

	if (sverr != EEXIST) {
		syslog(LOG_ERR,"$could not link \"%s\" to \"%s\"",tfn,fn);
		unlink(tfn);
		free(tfn);
		return 1;
	}

	if ((fp=fopen(fn,"r")) == NULL) {
		syslog(LOG_ERR,"$could not open existing lock file \"%s\"",fn);
		unlink(tfn);
		free(tfn);
		return 1;
	}

	fscanf(fp,"%d",&tmppid);
	pid=tmppid;
	fclose(fp);
	debug(4,"opened bsy file for pid %d",pid);
	if (kill(pid,0) && (errno == ESRCH)) {
		syslog(LOG_INFO,"found stale bsy file for %s, unlink",
			ascfnode(addr,0x1f));
		unlink(fn);
	} else {
		debug(4,"process active, lock failed");
		unlink(tfn);
		free(tfn);
		return 1;
	}

	if (link(tfn,fn) == 0) {
		debug(4,"created lock OK");
		unlink(tfn);
		free(tfn);
		return 0;
	} else {
		syslog(LOG_ERR,"$could not link \"%s\" to \"%s\"",tfn,fn);
		unlink(tfn);
		free(tfn);
		return 1;
	}
}

int nodeulock(addr)
faddr *addr;
{
	char *fn;
	FILE *fp;
	pid_t pid,mypid;
	int tmppid;

	debug(4,"try unlocking node %s",ascfnode(addr,0x1f));

	fn=bsyname(addr);
	if ((fp=fopen(fn,"r")) == NULL) {
		syslog(LOG_ERR,"$cannot open lock file \"%s\"",fn);
		return 1;
	}
	fscanf(fp,"%d",&tmppid);
	pid=tmppid;
	fclose(fp);
	mypid=getpid();

	if (pid == mypid) {
		debug(4,"unlinking lock file");
		unlink(fn);
		return 0;
	} else {
		syslog(LOG_ERR,"trying unlock file for process %u, we are %u",
			pid,mypid);
		return 1;
	}
}

#else /* don't HAVE_BSY */

int nodelock(addr)
faddr *addr;
{
	debug(4,"try locking node %s - no locking",ascfnode(addr,0x1f));
	return 0;
}

int nodeulock(addr)
faddr *addr;
{
	debug(4,"try unlocking node %s - no locking",ascfnode(addr,0x1f));
	return 0;
}

#endif /* HAVE_BSY */
